local Recount2 = Recount2
local tinsert = table.insert
local tconcat = table.concat
local bit_band = bit.band

local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end

local function FixCaps(capsstr)
	if type(capsstr)=="string" then
		return string.upper(string.sub(capsstr,1,1))..string.lower(string.sub(capsstr,2))
	else
		return nil
	end
end


--Each section consists of on left side what to select by and right side the actual details
local function ClickButton(self)
	self:GetParent():GroupOn(self, self.group, self.func)
end
local function CreateSectionButton(parent)
	local button = CreateFrame("FRAME", nil, parent)
	button:SetWidth(76)
	button:SetHeight(15)
	button.text = button:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	button.text:SetPoint("CENTER", button, "CENTER")
	button.text:SetTextColor(1,1,1,1)
	Recount2:AddFontString(button.text)
	SetFontSize(button.text, 13)
	
	button.background = button:CreateTexture(nil, "BACKGROUND")
	button.background:SetTexture(1, 0, 0 , 0.25)
	button.background:SetAllPoints(button)

	button.active = false

	button.Clicked = ClickButton
	button:SetScript("OnMouseDown", ClickButton)
	button:EnableMouse()

	return button
end

local function UpdateSectionButtons(self, groups, EntConstFuncs)
	local buttons = self.buttons

	for k, v in ipairs(groups) do
		if buttons[k]==nil then
			tinsert(buttons, CreateSectionButton(self))
			buttons[k].id = k
			buttons[k]:SetPoint("TOPLEFT", self, "TOPLEFT", 2, -2 - (k-1)*16)
		end
		buttons[k].text:SetText(v)
		buttons[k].group = v
		buttons[k].func = EntConstFuncs[k]
		buttons[k]:Show()
		buttons[k].background:SetTexture(1, 0, 0, 0.25)
		buttons[k].active = false
	end

	for i=#groups+1, #buttons do
		buttons[i]:Hide()
	end
end

local function UpdateSection(self, data, groups, EntConstFuncs)
	UpdateSectionButtons(self, groups, EntConstFuncs)

	self.buttons[1].background:SetTexture(0, 1, 0, 0.25)
	self.buttons[1].active = true

	self.data = data
	if self.groupedData then
		Recount2:FreeTableRecurseLimit(self.groupedData, 1)
	end
	self.groupedData = Recount2:GroupTables(data, groups[1])
	self.details:SetData(self.groupedData)
	self.details:SetEntryConstantFunc(EntConstFuncs[1])
	self.details:UpdateRows()
end

local function GroupOn(self, button, group, func)
	for _, b in ipairs(self.buttons) do
		b.background:SetTexture(1, 0, 0, 0.25)
		b.active = false
	end
	button.background:SetTexture(0, 1, 0, 0.25)
	button.active = true

	if self.groupedData then
		Recount2:FreeTableRecurseLimit(self.groupedData, 1)
	end
	self.groupedData = Recount2:GroupTables(self.data, group)
	self.details:SetData(self.groupedData)
	self.details:SetEntryConstantFunc(func)
	self.details:ClearSelection()
	self.details:UpdateRows()

	self:GetParent():OnGroupChange(self.id)
end

local function ClickSectionRow(passedArg, rowData)
	passedArg[1]:ClickedSection(passedArg[2], rowData)
end

local function CreateSection(parent, id)
	local section = CreateFrame("FRAME", nil, parent)

	detailTable = Recount2:CreateTableWindow(nil, section, 300, 100, 14, 1, 80, ClickSectionRow, {parent, id})
	section.details = detailTable	
	detailTable:ClearAllPoints()
	detailTable:SetPoint("TOPLEFT", section, "TOPLEFT", 80, 0)
	detailTable:SetPoint("BOTTOMRIGHT", section, "BOTTOMRIGHT")
	detailTable:SetNameTag("[ePos, eName:%d. %s]")
	detailTable:AddColumn("Min", "eMin", '[eMin:%d]', 'tMin', 45)
	detailTable:AddColumn("Average", "eAvg", '[eAvg:%d]', 'tAvg', 45)
	detailTable:AddColumn("Max", "eMax", '[eMax:%d]', 'tMax', 45)
	detailTable:AddColumn("Count", "eCount", "[eCount, 100*(eCount or 0)/tCount:%d (%.1f%%)]", 'tCount', 65)
	detailTable:AddColumn("Total", "eTotal", "[eTotal, 100*(eTotal or 0)/tTotal:%d (%.1f%%)]", 'tTotal', 75)
	
	section.buttons = Recount2:GetTable()
	section.id = id

	section.UpdateSection = UpdateSection
	section.GroupOn = GroupOn

	return section
end

local function SetData(self, data, groups)
	if #self.sections==0 then
		tinsert(self.sections, CreateSection(self, 1))
	end

	for _, v in ipairs(self.sections) do
		v:Hide()
	end

	self.groups = groups
	self.sections[1]:UpdateSection(data, groups, self.EntConstFuncs)
	self.sections[1]:ClearAllPoints()
	self.sections[1]:SetPoint("TOPLEFT", self.innerFrame, "TOPLEFT")
	self.sections[1]:SetPoint("BOTTOMRIGHT", self.innerFrame, "BOTTOMRIGHT")
	self.sections[1]:Show()

	self.sectionsShown = 1
end

local function UpdateMinHeight(self, sections)
	--Update Min Height required
	curHeight = (self.innerFrame:GetTop()-self.innerFrame:GetBottom())
	extraHeight = self:GetHeight()-curHeight
	minHeight = 114*self.sectionsShown
	self:SetMinResize(250, minHeight+extraHeight)

	--If less than min height update
	if curHeight<minHeight then
		self:SetHeight(minHeight+extraHeight)
		curHeight = minHeight
	end
end


local function OnGroupChange(self, sectionID)
	for k, v in ipairs(self.sections) do
		if k>sectionID then
			v:Hide()
		end
	end

	self.sectionsShown = sectionID

	--Update Min Height
	UpdateMinHeight(self)

	local sections = self.sections
	local h = curHeight/self.sectionsShown

	for k = 1, sectionID do
		sections[k]:ClearAllPoints()
		sections[k]:SetPoint("TOPLEFT", self.innerFrame, "TOPLEFT", 0, -(k-1)*h)
		sections[k]:SetPoint("BOTTOMRIGHT", self.innerFrame, "TOPRIGHT", 0, -k*h)
	end
end


local function ClickedSection(self, sectionID, rowData)
	local h = self.innerFrame:GetTop()-self.innerFrame:GetBottom()
	local groups = Recount2:GetTable()
	local groupFuncs = Recount2:GetTable()

	for k, v in ipairs(self.sections) do
		if k>(sectionID+1) then
			v:Hide()
		end
	end
	
	local sections = self.sections
	local section = sections[sectionID]
	for _, button in ipairs(section.buttons) do
		if button:IsShown() and not button.active then
			tinsert(groups, button.group)
			tinsert(groupFuncs, button.func)
		end
	end

	section.details:SetSelection(rowData)

	if #groups>0 then
		local nextSection = sectionID+1
		if sections[nextSection]==nil then
			tinsert(sections, CreateSection(self, nextSection))
		end

		sections[nextSection]:UpdateSection(rowData.tables, groups, groupFuncs)
		sections[nextSection]:Show()

		h = h/nextSection

		for k = 1, nextSection do
			sections[k]:ClearAllPoints()
			sections[k]:SetPoint("TOPLEFT", self.innerFrame, "TOPLEFT", 0, -(k-1)*h)
			sections[k]:SetPoint("BOTTOMRIGHT", self.innerFrame, "TOPRIGHT", 0, -k*h)
		end

		self.sectionsShown = nextSection

		--Update Min Height
		UpdateMinHeight(self)
	end

	Recount2:FreeTable(groups)
	Recount2:FreeTable(groupfuncs)
end

local function DetailResize(self)
	if self.sectionsShown>0 then
		local sections = self.sections
		local h = (self.innerFrame:GetTop()-self.innerFrame:GetBottom())/self.sectionsShown

		for k = 1, self.sectionsShown do
			sections[k]:ClearAllPoints()
			sections[k]:SetPoint("TOPLEFT", self.innerFrame, "TOPLEFT", 0, -(k-1)*h)
			sections[k]:SetPoint("BOTTOMRIGHT", self.innerFrame, "TOPRIGHT", 0, -k*h)
		end
	end
end



local function LoadsConstantsGUID(guid, data)
	local avg
	if data.total==nil then
		avg = nil
	else
		avg = data.total/(data.count or 1)
	end

	Recount2:AddExpressionConstants('eName', Recount2.Data.Combatants[guid].attrTable.name or "No One", 'eCount', data.count, 'eTotal', data.total, 'eAvg', avg, 'eMin', data.min, 'eMax', data.max)
end

local function LoadsConstantsAbility(ability, data)
	local avg
	if data.total==nil then
		avg = nil
	else
		avg = data.total/(data.count or 1)
	end

	if string.match(ability, "(%d+)") then
		name, rank = GetSpellInfo(tonumber(ability))
		if rank~="" then
			name = name.." ("..rank..")"
		end
		Recount2:AddExpressionConstants('eName', name, 'eCount', data.count, 'eTotal', data.total, 'eAvg', avg, 'eMin', data.min, 'eMax', data.max)
	else
		Recount2:AddExpressionConstants('eName', ability, 'eCount', data.count, 'eTotal', data.total, 'eAvg', avg, 'eMin', data.min, 'eMax', data.max)
	end
end


local function LoadsConstantsHitType(hitType, data)
	local name = Recount2:GetTable()
	for k, v in pairs(Recount2.GroupNames.Type) do
		if bit_band(k, hitType)==k then
			tinsert(name, v)
		end
	end

	local avg
	if data.total==nil then
		avg = nil
	else
		avg = data.total/(data.count or 1)
	end

	if #name>0 then
		Recount2:AddExpressionConstants('eName', tconcat(name, " and "), 'eCount', data.count, 'eTotal', data.total, 'eAvg', avg, 'eMin', data.min, 'eMax', data.max)
	else
		Recount2:AddExpressionConstants('eName', "Unknown Hit Type", 'eCount', data.count, 'eTotal', data.total, 'eAvg', avg, 'eMin', data.min, 'eMax', data.max)
	end
	Recount2:FreeTable(name)
end

local function LoadsConstantsSchool(school, data)
	local name = Recount2:GetTable()
	for k, v in pairs(Recount2.GroupNames.School) do
		if bit_band(k, school)==k then
			tinsert(name, v)
		end
	end

	local avg
	if data.total==nil then
		avg = nil
	else
		avg = data.total/(data.count or 1)
	end

	if #name>0 then
		Recount2:AddExpressionConstants('eName', tconcat(name, " and "), 'eCount', data.count, 'eTotal', data.total, 'eAvg', avg, 'eMin', data.min, 'eMax', data.max)
	else
		Recount2:AddExpressionConstants('eName', "Unknown School", 'eCount', data.count, 'eTotal', data.total, 'eAvg', avg, 'eMin', data.min, 'eMax', data.max)
	end
	Recount2:FreeTable(name)
end



local function LoadsConstantsClass(class, data)
	local avg
	if data.total==nil then
		avg = nil
	else
		avg = data.total/(data.count or 1)
	end

	Recount2:AddExpressionConstants('eName', FixCaps(class) or "Unknown", 'eCount', data.count, 'eTotal', data.total, 'eAvg', avg, 'eMin', data.min, 'eMax', data.max)
end



local function ClickedRow(self)
	if self.dataTable then
		Recount2:UpdateDetailWindow("Test", self.dataTable)
	end
end


function Recount2:UpdateDetailWindow(name, title, data)
	local window = Recount2.detailWindows[name]

	
	groups =  { "Ability", "Source Class", "Target Class", "Hit Type", "School", "Source GUID", "Target GUID"}
	entConstFuncs = {LoadsConstantsAbility, LoadsConstantsClass, LoadsConstantsClass, LoadsConstantsHitType, LoadsConstantsSchool, LoadsConstantsGUID, LoadsConstantsGUID}

	if window == nil then
		window=Recount2.Widgets:CreateWindow("Recount2_DetailWindow_"..name, 400, 200)
		window:SetPoint("TOP", UIParent, "TOP")

		curHeight = (window.innerFrame:GetTop()-window.innerFrame:GetBottom())
		extraHeight = window:GetHeight()-curHeight
		minHeight = 114
		window:SetMinResize(250, minHeight+extraHeight)

		window.sections = Recount2:GetTable()
		window.ClickedSection = ClickedSection
		window.OnSizeChanged = DetailResize
		window.OnGroupChange = OnGroupChange
		window.sectionsShown = 0 

		Recount2.detailWindows[name] = window

		self.data = Recount2:GetTable()
	end

	local dataCopy = self.data
	
	for k in pairs(dataCopy) do
		dataCopy[k] = nil
	end

	for k, v in pairs(data) do
		dataCopy[k] = v
	end

	window:SetTitle(title)
	window.EntConstFuncs = entConstFuncs
	window:SetScript("OnSizeChanged", window.OnSizeChanged)

	SetData(window, dataCopy, groups)

	Recount2:SetWindowTop(window)
	window:Show()
end

