local Recount2 = Recount2
local Recount2_Widgets = Recount2.Widgets
local tinsert = table.insert
local tremove = table.remove
local tsort = table.sort
local math_floor = math.floor

local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end

local function entrySort(a, b)
	if a[1]==b[1] then
		return a[2]<b[2]
	end
	return a[1]<b[1]
end

local function AddEntry(self, text, value)
	--Just creating table here since the table cache doesn't really have an issue being at 0
	tinsert(self.entries, {text, value})
	--tsort(self.entries, entrySort)
	self.selected = 0
	self:UpdateRows()
end

local function RemoveEntryByText(self, text)
	for k, v in ipairs(self.entries) do
		if v[1]==text then
			self.selected = 0
			tremove(self.entries, k)
			self:UpdateRows()
			return
		end
	end
end

local function RemoveEntryByValue(self, value)
	for k, v in ipairs(self.entries) do
		if v[2]==value then
			self.selected = 0
			tremove(self.entries, k)
			self:UpdateRows()
			return
		end
	end
end

local function RemoveEntries(self)
	for k, v in ipairs(self.entries) do
		self.entries[k] = nil
	end
	self:UpdateRows()
end

local function GetSelected(self)
	if self.selected and self.entries[self.selected] then
		return self.entries[self.selected][2]
	end
end

local function rowClick(self)
	if self.clickFunc then
		self.clickFunc(self.arg, self.id)
	end
end

local function AddRow(parent, width, height, id, clickFunc, arg)
	local row = CreateFrame("FRAME", nil, parent)

	row:SetWidth(width)
	row:SetHeight(height)
	row.id = id
	row.arg = arg
	row.clickFunc = clickFunc

	row.selected = row:CreateTexture(nil, "BACKGROUND")
	row.selected:SetAllPoints(row)
	row.selected:SetTexture(1, 1, 0 , 0.25)
	row.selected:Hide()

	row.text=row:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	row.text:SetPoint("LEFT", row, "LEFT", 2, 0)
	row.text:SetPoint("RIGHT", row, "RIGHT", -2, 0)
	row.text:SetJustifyH("LEFT")
	row.text:SetText("")
	row.text:SetTextColor(1,1,1,1)
	Recount2:AddFontString(row.text)
	SetFontSize(row.text, height*0.8)

	row:EnableMouse()
	row:SetScript("OnMouseDown", rowClick)

	return row
end

local function OnSizeChanged(self)
	local w = self:GetRight()-self:GetLeft()
	local h = self:GetTop()-self:GetBottom()

	if h~=self.lastHeight then
		self.lastHeight = h

		local numRows = (h-2+self.rowSpacing)/(self.rowHeight+self.rowSpacing)

		local rowCount = #self.rows

		if numRows>rowCount then
			--Create the new rows needed
			for i = rowCount+1, numRows do
				local row = AddRow(self, w, self.rowHeight, i, self.clickRow, self)
				tinsert(self.rows, row)
				row:SetPoint("TOPLEFT", self, "TOPLEFT", 0, -(i-1)*(self.rowHeight+self.rowSpacing))
			end
		elseif numRows<self.numRows then
			--Hide any extra rows that are visible
			for i = numRows+1, self.numRows do
				self.rows[i]:Hide()
			end
		end

		self.numRows = numRows

		self:UpdateRows()
	end

	if w~=self.lastWidth then
		self.lastWidth = w
		for _, row in ipairs(self.rows) do
			row:SetWidth(w)
		end
	end
end

local function clickRow(self, id)
	if self.vScroll:IsShown() then
		self.selected = id + math_floor(self.vScroll:GetValue()+0.5)
	else
		self.selected = id
	end
	self:UpdateRows()
end

local function UpdateRows(self)
	local offset = 0
	if #self.entries>self.numRows then
		self.vScroll:SetMinMaxValues(0, (#self.entries-self.numRows))

		if not self.vScroll:IsShown() then
			self.vScroll:SetValue(0)
			self.vScroll:Show()

			for _, row in ipairs(self.rows) do
				row:SetWidth(self.lastWidth-16-2)
			end
		end

		offset = math_floor(self.vScroll:GetValue()+0.5)
	elseif self.vScroll:IsShown() then
		for _, row in ipairs(self.rows) do
			row:SetWidth(self.lastWidth-2)
		end

		self.vScroll:Hide()
	end

	for i = 1, self.numRows do
		local row = self.rows[i]
		if self.entries[i+offset] then
			row.text:SetText(self.entries[i+offset][1])
			row:Show()

			if self.selected == i+offset then
				row.selected:Show()
			else
				row.selected:Hide()
			end
		else
			row:Hide()
		end
	end
end

local function OnMouseWheel(self, arg1)
	if arg1==1 then
		self.vScroll:Decrease()
	elseif arg1==-1 then
		self.vScroll:Increase()
	end
end

function Recount2_Widgets:CreateScrollList(parent, width, height, rowHeight, rowSpacing)
	local scrollList = CreateFrame("FRAME", nil, parent)

	scrollList.entries = {}
	scrollList.rows = {}

	scrollList:SetWidth(width)
	scrollList:SetHeight(height)

	scrollList.lastHeight = height
	scrollList.lastWidth = width

	scrollList.rowHeight = rowHeight
	scrollList.rowSpacing = rowSpacing

	scrollList.UpdateRows = UpdateRows
	scrollList.clickRow = clickRow
	scrollList.OnSizeChanged = OnSizeChanged
	scrollList.AddEntry = AddEntry
	scrollList.RemoveEntryByText = RemoveEntryByText
	scrollList.RemoveEntryByValue = RemoveEntryByValue
	scrollList.RemoveEntries = RemoveEntries
	scrollList.GetSelected = GetSelected
	scrollList.OnMouseWheel = OnMouseWheel

	scrollList.selected = 0

	scrollList:SetScript("OnSizeChanged", scrollList.OnSizeChanged)
	scrollList:SetScript("OnMouseWheel", scrollList.OnMouseWheel)
	scrollList:EnableMouseWheel()

	Recount2_Widgets:AddBorder(scrollList)



	scrollList.numRows = (height-2+rowSpacing)/(rowHeight+rowSpacing)

	for i = 1, scrollList.numRows do
		local row = AddRow(scrollList, width - 2, rowHeight, i, scrollList.clickRow, scrollList)
		tinsert(scrollList.rows, row)
		row:SetPoint("TOPLEFT", scrollList, "TOPLEFT", 1, -(i-1)*(rowHeight+rowSpacing)-1)
	end

	scrollList.vScroll = Recount2_Widgets:CreateScrollbar(scrollList, 16, height-2, "VERTICAL")
	scrollList.vScroll:SetPoint("RIGHT", scrollList, "RIGHT", -1, 0)
	scrollList.vScroll:Hide()
	scrollList.vScroll:SetOnChange(scrollList.UpdateRows, scrollList)

	return scrollList
end
